package com.miyzh.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.miyzh.dao.ClinicDao;
import com.miyzh.dao.MedicineStorageDao;
import com.miyzh.dao.MedicineStoreDao;
import com.miyzh.entity.Clinic;
import com.miyzh.entity.MedicineStorage;
import com.miyzh.entity.MedicineStore;
import com.miyzh.entity.dto.MedicineStoreDto;
import com.miyzh.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 库存信息同步
 */

@Slf4j
@Service
public class MedicineStorageService {

    @Autowired
    private MedicineStorageDao medicineStorageDao;
    @Value ("${postApi.storeUrl}")
    private String url;
    @Value("${postApi.pageSize}")
    private int pageSize;

    @Async
    public void uploadMedicine(List<Clinic> clinicList) throws InterruptedException, JsonProcessingException {
        long startTime = System.currentTimeMillis();
        for(Clinic clinic : clinicList) {
            PageInfo<MedicineStorage> page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                medicineStorageDao.listMedicinePurchase(clinic.getClinicId());
            });
            Map<String, List<MedicineStorage>> map =  page.getList().stream()
                    .collect(Collectors.groupingBy(MedicineStorage::getStoreStorageCode, Collectors.toList()));
//            storeInfo.setMedicineInfos(page.getList());
//            this.upload(storeInfo);
//            if (page.getPages() > 1) {
//                for(int i = 2; i <= page.getPages(); i++) {
//                    PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
//                        medicineStoreDao.listMedicineStore(clinic.getClinicId());
//                    });
//                    storeInfo.setMedicineInfos(page.getList());
//                    this.upload(storeInfo);
//                }
//            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("1方法执行用时--{}", startTime - syncTime);
    };

    @Async
    public void uploadChinaMedicine(List<Clinic> clinicList) throws JsonProcessingException {
        long startTime = System.currentTimeMillis();
        MedicineStoreDto storeInfo = new MedicineStoreDto();
        storeInfo.setCheckMan("");
        storeInfo.setCheckTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd hh:mm:ss")));
        for(Clinic clinic : clinicList) {
            storeInfo.setStoreCode(clinic.getStoreCode());
            PageInfo<MedicineStore> page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                medicineStorageDao.listMedicinePurchase(clinic.getClinicId());
            });
            storeInfo.setMedicineInfos(page.getList());
            this.upload(storeInfo);
            if (page.getPages() > 1) {
                for(int i = 2; i <= page.getPages(); i++) {
                    PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                        medicineStorageDao.listMedicinePurchase(clinic.getClinicId());
                    });
                    storeInfo.setMedicineInfos(page.getList());
                    this.upload(storeInfo);
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("2方法执行用时--{}", startTime - syncTime);
    };

    public void upload (Object o) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        String params = objectMapper.writeValueAsString(o);
            System.out.println(objectMapper.writeValueAsString(RestTemplateUtil.postHttp(url, params)));
    }

}
