package com.miyzh.service;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.miyzh.dao.ClinicDao;
import com.miyzh.dao.MedicineStoreDao;
import com.miyzh.entity.BaseResult;
import com.miyzh.entity.Clinic;
import com.miyzh.entity.MedicineStore;
import com.miyzh.entity.dto.MedicineStoreDto;
import com.miyzh.utils.RestTemplateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * 库存信息同步
 */

@Slf4j
@Service
public class MedicineStoreService {

    @Autowired
    private ClinicDao clinicDao;
    @Autowired
    private MedicineStoreDao medicineStoreDao;
    @Value ("${postApi.storeUrl}")
    private String url;
    @Value("${postApi.pageSize}")
    private int pageSize;

    @Async
    public void uploadMedicine(List<Clinic> clinicList) throws InterruptedException, JsonProcessingException {
        long startTime = System.currentTimeMillis();
        MedicineStoreDto storeInfo = new MedicineStoreDto();
        storeInfo.setCheckMan("");
        storeInfo.setCheckTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd hh:mm:ss")));
        for(Clinic clinic : clinicList) {
            storeInfo.setStoreCode(clinic.getStoreCode());
            PageInfo<MedicineStore> page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                medicineStoreDao.listMedicineStore(clinic.getClinicId());
            });
            storeInfo.setMedicineInfos(page.getList());
            this.upload(storeInfo);
            if (page.getPages() > 1) {
                for(int i = 2; i <= page.getPages(); i++) {
                    PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                        medicineStoreDao.listMedicineStore(clinic.getClinicId());
                    });
                    storeInfo.setMedicineInfos(page.getList());
                    this.upload(storeInfo);
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("uploadMedicine方法执行用时--{}", startTime - syncTime);
    };

    @Async
    public void uploadChinaMedicine(List<Clinic> clinicList) throws JsonProcessingException {
        long startTime = System.currentTimeMillis();
        MedicineStoreDto storeInfo = new MedicineStoreDto();
        storeInfo.setCheckMan("");
        storeInfo.setCheckTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd hh:mm:ss")));
        for(Clinic clinic : clinicList) {
            storeInfo.setStoreCode(clinic.getStoreCode());
            PageInfo<MedicineStore> page = PageHelper.startPage(1, pageSize).doSelectPageInfo(() -> {
                medicineStoreDao.listChinaMedicineStore(clinic.getClinicId());
            });
            storeInfo.setMedicineInfos(page.getList());
            this.upload(storeInfo);
            if (page.getPages() > 1) {
                for(int i = 2; i <= page.getPages(); i++) {
                    PageHelper.startPage(i, pageSize).doSelectPageInfo(() -> {
                        medicineStoreDao.listChinaMedicineStore(clinic.getClinicId());
                    });
                    storeInfo.setMedicineInfos(page.getList());
                    this.upload(storeInfo);
                }
            }
        };
        long syncTime = System.currentTimeMillis();
        log.info("uploadChinaMedicine方法执行用时--{}", startTime - syncTime);
    };

    public void upload (MedicineStoreDto storeInfo) throws JsonProcessingException {
        ObjectMapper objectMapper = new ObjectMapper();
        String params = objectMapper.writeValueAsString(storeInfo);
        BaseResult baseResult = RestTemplateUtil.postHttp(url, params);
        if (baseResult.isSuccess()) {
            log.info(storeInfo.getStoreCode() + baseResult.getMessage());
        } else {
            log.error(storeInfo.getStoreCode() + baseResult.getMessage());
        }


    }

}
